%==========================================================================
% Spatial Dynamic Panel Data Models with Interactive Fixed Effects
% Last updated: 2017/6/28
% This program implements the models in Shi and Lee (2017) "Spatial Dynamic
% Panel Data Models with Interactive Fixed Effects", Journal of 
% Econometrics, 2017(197), 323-347.
% This code is offered with no guarantee. I appreciate for any feedback on
% issues.
% 
%==========================================================================
% Author
%   Wei Shi, Institute for Economic and Social Research, Jinan University
%   Email: shiweiemail@gmail.com
%
%==========================================================================
% Method
% The profile objective function (Eq.5) is maximized with respect to 
% spatial interaction coefficients and regression coefficients using
% fminunc.
%
%==========================================================================
% Input parameters
%   Y: NxT matrix of outcomes
%   W: NxN spatial weights matrix in the outcome equation
%   W_R: NxN spatial weights matrix in the errors
%   X: KxNxT regressors
%   R: number of factors
%   start: initial values
%   TolX: stoping criteria on beta
%
% Output parameters
%   beta: QML estimate with entries corresponding to (see Eq.1 for model
%       specification)
%       beta(1): Y(t-1): gamma
%       beta(2): W*Y(t-1): rho
%       beta(3:K+2): X
%       beta(K+3): W*Y: lambda (spatial interaction coefficient)
%       beta(K+4): alpha: spatial error
%   exitflag: Positive exit flags correspond to successful outcomes.
%   gamma: NxR factor loadings
%   f: TxR time factors
%   Vbeta: estimated variance-covariance matrix
%   bcorr: bias correction term (see Section 3.3 of the paper)
%       the bias corrected estimator is beta-bcorr

function [beta,bcorr,Vbeta,gamma,f,exitflag,stats,resid] = SFactors(Y,W,W_R,X,R,start,TolX)
    N = size(Y,1);
    T = size(Y,2);
    K = size(X,1)-2;
    npara = K+4; % parameters: lambda, gamma, rho, alpha, beta
    Y_1 = squeeze(X(1,:,:));
    WY_1 = squeeze(X(2,:,:));      
    % JPE changed 100000 into 500000 below    
    opt = optimoptions('fminunc','TolX',TolX,'MaxFunEvals',500000,'MaxIter',500000,'Display','notify','Algorithm','quasi-newton');
    [beta,~,exitflag] = fminunc(@Q_obj,start,opt,Y,W,W_R,X,R);
        
    lambda_e = (exp(2*beta(K+3))-1)/(exp(2*beta(K+3))+1);
    gamma_e  = (exp(2*beta(1))-1)/(exp(2*beta(1))+1);
    rho_e    = (exp(2*beta(2))-1)/(exp(2*beta(2))+1);
    alpha_e  = (exp(2*beta(K+4))-1)/(exp(2*beta(K+4))+1);
    beta_x   = beta(3:(K+2));
    Xsum     = zeros(N,T);
    for k = 1:K
        Xsum = Xsum + beta_x(k)*squeeze(X(2+k,:,:));
    end
    Sn = eye(N)-lambda_e*W;
    Rn = eye(N)-alpha_e*W_R;
    G = W/Sn;
    GT = W_R/Rn;
    Xlambda = G*(Y_1*gamma_e+WY_1*rho_e+Xsum);

    M = Rn*(Sn*Y-Y_1*gamma_e-WY_1*rho_e-Xsum)/(sqrt(N*T));
    [U,D,~] = svd(M);
    D = diag(D).*diag(D);
    [D,I] = sort(D);
    Gamma_e = zeros(N,R);
    for c = 1:R
        Gamma_e(:,c) = U(:,I(end-c+1));
    end
    P_Ge = Gamma_e*Gamma_e';
    M_Ge = eye(N) - P_Ge;
    sigma_sq = sum(D(1:(end-R)))

    [U,D,~] = svd(M');
    D = diag(D).*diag(D);
    [~,I] = sort(D);
    F_e = zeros(T,R);
    for c = 1:R
        F_e(:,c) = U(:,I(end-c+1));
    end
    P_Fe = F_e*F_e';
    M_Fe = eye(T) - P_Fe;

    % [gamma, rho, beta, lambda, alpha]
    D = zeros(npara,npara);    
    D(1,1) = trace(M_Ge*Rn*Y_1*M_Fe*Y_1'*Rn')/(N*T*sigma_sq);   % gamma
    D(1,2) = trace(M_Ge*Rn*Y_1*M_Fe*WY_1'*Rn')/(N*T*sigma_sq);  % rho
    for j = 3:(2+K)
        D(1,j) = trace(M_Ge*Rn*Y_1*M_Fe*squeeze(X(j,:,:))'*Rn')/(N*T*sigma_sq);    % beta_x
    end
    D(1,3+K) = trace(M_Ge*Rn*Y_1*M_Fe*Xlambda'*Rn')/(N*T*sigma_sq);    % lambda

    D(2,1) = D(1,2);
    D(2,2) = trace(M_Ge*Rn*WY_1*M_Fe*WY_1'*Rn')/(N*T*sigma_sq);
    for j = 3:(2+K)
        D(2,j) = trace(M_Ge*Rn*WY_1*M_Fe*squeeze(X(j,:,:))'*Rn')/(N*T*sigma_sq);    % beta_x
    end    
    D(2,3+K) = trace(M_Ge*Rn*WY_1*M_Fe*Xlambda'*Rn')/(N*T*sigma_sq);

    for i = 3:(2+K)
        D(i,1) = D(1,i);
        D(i,2) = D(2,i);
        for j = 3:(2+K)
            D(i,j) = trace(M_Ge*Rn*squeeze(X(i,:,:))*M_Fe*squeeze(X(j,:,:))'*Rn')/(N*T*sigma_sq);    % beta_x
        end
        D(i,3+K) = trace(M_Ge*Rn*squeeze(X(i,:,:))*M_Fe*Xlambda'*Rn')/(N*T*sigma_sq);
    end        
    
    for j = 1:(2+K)
        D(3+K,j) = D(j,3+K);
    end
    D(3+K,3+K) = trace(M_Ge*Rn*Xlambda*M_Fe*Xlambda'*Rn')/(N*T*sigma_sq)+trace(Rn*G/Rn/Rn'*G'*Rn')/N+trace(G*G)/N-2*(trace(G)/N)^2;
    D(3+K,4+K) = trace(G*GT)/N+trace(Rn*G/Rn*GT')/N-trace(G)*trace(GT)*2/N^2;
    D(4+K,3+K) = D(3+K,4+K);
    D(4+K,4+K) = trace(GT*GT')/N+trace(GT*GT)/N-2*(trace(GT)/N)^2;

    % parameter vector: gamma;rho;beta;lambda;alpha
    phi             = zeros(4+K,1);
    J0              = [zeros(T,T) eye(T,T)]';
    A               = Sn\(gamma_e*eye(N,N)+rho_e*W);
    for h = 1:(T-1)
        Jh          = [zeros(T,T-h) eye(T,T) zeros(T,h)]';
        phi(1,1)    = phi(1,1) + 1/(sqrt(N*T))*trace(J0*P_Fe*Jh')*trace(A^(h-1)/Sn);
        phi(2,1)    = phi(2,1) + 1/(sqrt(N*T))*trace(J0*P_Fe*Jh')*trace(W*A^(h-1)/Sn);
        phi(3+K,1)    = phi(3+K,1) + 1/(sqrt(N*T))*trace(J0*P_Fe*Jh')*trace(G*(gamma_e*eye(N,N)+rho_e*W)*A^(h-1)/Sn);
    end
    
    x = zeros(4+K,1);
    x(3+K,1) = sqrt(T/N)*(trace(G)*R/N-trace(P_Ge*Rn*G/Rn));
    x(4+K,1) = sqrt(T/N)*(trace(GT)*R/N-trace(P_Ge*GT));
    x = x-phi;
    bcorr = (D\x)/sqrt(N*T);
    beta = [gamma_e;rho_e;beta_x;lambda_e;alpha_e];
    Vbeta = inv(D)/(N*T);
 
    % factor loadings and factors
    gamma = Gamma_e;
    f = ((gamma'*gamma)\(gamma'*(Rn*(Sn*Y-Y_1*gamma_e-WY_1*rho_e-Xsum))))';
    
    % Next block of lines added by J.P.Elhorst, 22-11-2017
    %
    % R-squared + residuals
    %
    resid=Rn*(Sn*Y-Y_1*gamma_e-WY_1*rho_e-Xsum)-gamma*f';
    Xsum1     = zeros(N,T);
    for k = 1:K
        Xsum1 = Xsum1 + (beta_x(k)-bcorr(k))*squeeze(X(2+k,:,:));
    end
    Sn1 = eye(N)-(lambda_e-bcorr(end-1))*W;
    Rn1 = eye(N)-(alpha_e-bcorr(end))*W_R;
    resid1=Rn1*(Sn1*Y-Y_1*(gamma_e-bcorr(1))-WY_1*(rho_e-bcorr(2))-Xsum1)-gamma*f';
    yv=reshape(Y,N*T,1);
    residv=reshape(resid,N*T,1);
    residv1=reshape(resid1,N*T,1);
    yme=yv-mean(yv);
    rsqr1=residv'*residv;
    rsqr11=residv1'*residv1;
      rsqr2=yme'*yme;
    stats(1,1)=1.0-rsqr1/rsqr2;
    stats(2,1)=1.0-rsqr11/rsqr2;
%     res1=yme;
%     res2=Yhat-mean(yv);
%     res21=Yhat1-mean(yv);
%     rsqr(3,1)=(res1'*res2)^2/((res1'*res1)^2*(res2'*res2)^2);
%     rsqr(4,1)=(res1'*res21)^2/((res1'*res1)^2*(res21'*res21)^2);
    % 
    % LogLikelihood
    %
    tmp2 = 1/(2*sigma_sq);
    LogL=-(N*T/2)*log(pi) - (N*T/2)*log(sigma_sq) + T*log(det(Sn*Rn)) - tmp2*rsqr1;
    LogL1=-(N*T/2)*log(pi) - (N*T/2)*log(sigma_sq) + T*log(det(Sn1*Rn1)) - tmp2*rsqr11;
    stats(3,1)=LogL;
    stats(4,1)=LogL1;
    stats(5,1)=sigma_sq;
return

function f = Q_obj(theta,Y,W,W_R,X,R)
    % This function evaluates the likelihood function Q_nT(theta), see
    % Eq.5.
    % R factors are concentrated out.
    N = size(Y,1);
    T = size(Y,2);
    K = size(X,1)-2;    
        
    lambda = (exp(2*theta(K+3))-1)/(exp(2*theta(K+3))+1);
    gamma  = (exp(2*theta(1))-1)/(exp(2*theta(1))+1);
    rho    = (exp(2*theta(2))-1)/(exp(2*theta(2))+1);
    alpha  = (exp(2*theta(K+4))-1)/(exp(2*theta(K+4))+1);
    R_n    = eye(N)-alpha*W_R;
    S_n    = eye(N)-lambda*W;

    Xsum = zeros(N,T);
    for k = 1:K
        Xsum = Xsum + theta(2+k)*squeeze(X(2+k,:,:));
    end
    Y_1 = squeeze(X(1,:,:));
    WY_1 = squeeze(X(2,:,:));    
    M = R_n*(S_n*Y-Y_1*gamma-WY_1*rho-Xsum);
    
    if sum(sum(isnan(M))) > 0 || sum(sum(isinf(M)))
        f = 4.503e+15;
    else    
        [~,D,~] = svd(M);
        D = diag(D).*diag(D);
        D = sort(D);
        f = -(1/N)*log(det(S_n))-(1/N)*log(det(R_n))+(1/2)*log(sum(D(1:end-R))/(N*T));
    end
return